<template>
    <div class="app-data-chart <#if ctrl.getPSSysCss()??><#assign singleCss = ctrl.getPSSysCss()> ${singleCss.getCssName()}</#if>">
<#if ctrl.render??><#t>
        ${ctrl.render.code}
<#else><#t>
        <div class="app-charts" :id="chartId" style="<#if ctrl.getWidth() gt 0>width: ${ctrl.getWidth()};</#if>height: <#if ctrl.getHeight() gt 0>${ctrl.getHeight()}px<#else>100%</#if>;padding: 6px 0;"></div>
</#if>
  </div>
</template>

<#assign import_block>
import echarts from 'echarts';
import CodeListService from "@service/app/codelist-service";
import { ChartDataSetField,ChartLineSeries,ChartFunnelSeries,ChartPieSeries,ChartBarSeries} from '@/model/chart-detail';
</#assign>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_HEADER-BASE.vue.ftl
</#ibizinclude>

    /**
     * 获取多项数据
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getDatas(): any[] {
        return [];
    }

    /**
     * 获取单项树
     *
     * @returns {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getData(): any {
        return null;
    }

    /**
     * 显示处理提示
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop({ default: true }) protected showBusyIndicator!: boolean;

    /**
     * 部件行为--fetch
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected fetchAction!: string;  

    /**
    * Vue声明周期(组件初始化完毕)
    *
    * @memberof ${srfclassname('${ctrl.codeName}')}
    */
    public created() {
         this.afterCreated();     
    }

    /**
    * 执行created后的逻辑
    *
    * @memberof ${srfclassname('${ctrl.codeName}')}
    */
    public afterCreated(){
        if (this.viewState) {
            this.viewStateEvent = this.viewState.subscribe(({ tag, action, data }) => {
                if (!Object.is(tag, this.name)) {
                    return;
                }
                if (Object.is('load', action)) {
                    this.load(data);
                }
            });
        }  
    }

    /**
     * vue 生命周期
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected destroyed() {
        this.afterDestroy();
    }

    /**
     * 执行destroyed后的逻辑
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected afterDestroy() {
        if (this.viewStateEvent) {
            this.viewStateEvent.unsubscribe();
        }
        <#if destroyed_block??>
        ${destroyed_block}
        </#if>
    }

    /**
     * 图表div绑定的id
     *
     * @type {}
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */   
    public chartId:string = this.$util.createUUID();

    /**
     * echarts图表对象
     *
     * @type {}
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */   
    public myChart:any;

    /**
     * 代码表服务对象
     *
     * @type {CodeListService}
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */  
    public codeListService:CodeListService = new CodeListService({ $store: this.$store });

    /**
     * 序列模型
     *
     * @type {}
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public  seriesModel:any = {
        <#if ctrl.getPSDEChartSerieses()??>
        <#list ctrl.getPSDEChartSerieses() as chartSeries>
        ${chartSeries.getName()?lower_case}:${P.getPartCode(chartSeries, 'SERIES_MODEL').code}<#if chartSeries_has_next>,</#if>
        </#list>
        </#if>
    };

    /**
     * 图表自定义参数集合
     *
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */   
    public chartUserParams:any ={
        <#if ctrl.getUserParamNames()??>
        <#list ctrl.getUserParamNames() as userparam>
        <#if userparam?index_of("EC.")==0>
        ${userparam?remove_beginning("EC.")}:${ctrl.getUserParam(userparam)}<#if userparam_has_next>,</#if>
        </#if>
        </#list>
        </#if>
    };

    /**
     * 初始化图表所需参数
     *
     * @type {}
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */   
    public chartOption:any = {
        <#-- 标题start -->
        <#if ctrl.getPSDEChartTitle()??>
        <#assign title= ctrl.getPSDEChartTitle() />
        title:{
            show:<#if title.isShowTitle()>true<#else>false</#if> ,
            text:'<#if title.getTitle()??>${title.getTitle()}</#if>',
            subtext:'<#if title.getSubTitle()??>${title.getSubTitle()}</#if>'
        },
        </#if>
        <#-- 标题end -->
        <#-- 图例start -->
        <#if ctrl.getPSDEChartLegend()??>
        <#assign legend= ctrl.getPSDEChartLegend() />
        legend:{
            show:<#if legend.isShowLegend()>true<#else>false</#if>
        },
        </#if>
        <#-- 图例end -->
        <#-- 绘图网格start -->
        <#if ctrl.getPSChartGrids()??>
        grid:[
            <@ibizindent blank=8>
            <#list ctrl.getPSChartGrids() as chartGrid>
            ${P.getPartCode(chartGrid).code}<#if chartGrid_has_next>,</#if>
            </#list>
            </@ibizindent>
        ],
        </#if>
        <#-- 绘图网格end -->
        <#-- X轴start -->
        <#if ctrl.getPSChartXAxises()??>
        xAxis: [
            <@ibizindent blank=8>
            <#list ctrl.getPSChartXAxises() as chartXAxise>
            ${P.getPartCode(chartXAxise).code}<#if chartXAxise_has_next>,</#if>
            </#list>
            </@ibizindent>
        ],
        </#if>
        <#-- X轴end -->
        <#-- Y轴start -->
        <#if ctrl.getPSChartYAxises()??>
        yAxis:[
            <@ibizindent blank=8>
            <#list ctrl.getPSChartYAxises() as chartYAxise>
            ${P.getPartCode(chartYAxise).code}<#if chartYAxise_has_next>,</#if>
            </#list>
            </@ibizindent>
        ],
        </#if>
        <#-- Y轴end -->
        <#-- 提示框组件start -->
        tooltip:{
            show:true
        },
        <#-- 提示框组件end -->
        <#-- 数据集start -->
        dataset:[],
        <#-- 数据集end -->
        <#-- 序列start -->
        series:[
            <@ibizindent blank=8>
            <#list ctrl.getPSDEChartSerieses() as chartSeriese>
             ${P.getPartCode(chartSeriese).code}<#if chartSeriese_has_next>,</#if>
            </#list>
            </@ibizindent>
        ]
        <#-- 序列end -->
    };

    /**
     * 刷新
     *
     * @param {*} [opt={}]
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    protected refresh(opt: any = {}) {
        this.load(opt);
    }

    /**
     * 获取图表数据
     * 
     * @returns {*} 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public load(opt?:any) {
        let _this = this;
        const arg: any = { ...opt };
        const parentdata: any = {};
        this.$emit('beforeload', parentdata);
        Object.assign(arg, parentdata);
        Object.assign(arg,{viewparams:this.viewparams});
        this.service.search(this.fetchAction,JSON.parse(JSON.stringify(this.context)),arg,this.showBusyIndicator).then((res) => {
            if (res) {
               this.transformToBasicChartSetData(res.data,() =>{_this.drawCharts()});
            }
        }).catch((error) => {
            console.error(error);
        });
    }

    /**
     * 绘制图表
     * 
     * @returns {*} 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public drawCharts(){
        if(!this.myChart){
          let element:any =  document.getElementById(this.chartId);
          this.myChart = echarts.init(element);
        }
        this.handleChartOPtion();
        console.log(this.chartOption);
        this.myChart.setOption(this.chartOption);
        this.myChart.resize();
    }

    /**
     * 处理图表参数
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public handleChartOPtion(){
        if(Object.keys(this.seriesModel).length > 0){
            let tempDataSourceMap:Map<string,any> = new Map();
            for(let i=0;i<Object.keys(this.seriesModel).length;i++){
                Object.values(this.seriesModel).forEach((seriesvalue:any) =>{
                    if(seriesvalue.seriesIndex === i){
                        tempDataSourceMap.set(seriesvalue.name,seriesvalue.data);
                    }
                })
            }
            if(tempDataSourceMap.size > 0){
                tempDataSourceMap.forEach((item:any) =>{
                    this.chartOption.dataset.push({'source':item});
                })
            }
            console.log(this.chartOption);
            Object.keys(this.seriesModel).forEach((seriesName:string) =>{
                if(this.chartOption && this.chartOption.series.length > 0){
                    this.chartOption.series.forEach((item:any) =>{
                        if(this.seriesModel[seriesName].ecxObject){
                            item = Util.deepObjectMerge(item,this.seriesModel[seriesName].ecxObject);
                        }
                        if(this.seriesModel[seriesName].ecObject){
                            item = Util.deepObjectMerge(item,this.seriesModel[seriesName].ecObject);
                        }
                    })
                }
                //设置多序列
                let tempSeries:any = this.seriesModel[seriesName];
                if(tempSeries && tempSeries.seriesIdField && tempSeries.seriesValues.length > 0){
                    const returnIndex:number = this.chartOption.series.findIndex((item:any) =>{
                        return Object.is(item.id,seriesName);
                    })
                    this.chartOption.series.splice(returnIndex,1);
                    let tempSeriesArray:Array<any> = [];
                    tempSeries.seriesValues.forEach((seriesvalueItem:any) =>{
                        let tempSeriesTemp:any = JSON.parse(JSON.stringify(tempSeries.seriesTemp));
                        <#noparse>Object.assign(tempSeriesTemp,{name:tempSeries.seriesMap[seriesvalueItem],datasetIndex:tempSeries.seriesIndex,encode:{x:tempSeries.categorField,y:`${seriesvalueItem}`}});</#noparse>
                        this.chartOption.series.push(tempSeriesTemp);
                    })
                    

                }
            })
        }
        if(Object.keys(this.chartUserParams).length >0){
            Object.assign(this.chartOption,this.chartUserParams);
        }
    }

    /**
     * 实体数据集转化为图表数据集
     *    
     * @param {*} data 实体数据集
     * @param {Function} callback 回调
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public async transformToBasicChartSetData(data:any,callback:Function){
        if(!data || !Array.isArray(data) || data.length === 0){
            return;
        }
        //获取代码表值
        let allCodeList:any = await this.getChartAllCodeList();
        if(Object.values(this.seriesModel).length > 0){
            Object.values(this.seriesModel).forEach((singleSeries:any,index:number) =>{
                let tempChartSetData:Array<any> = [];
                let tempSeriesValues:Map<string,any> = new Map();
                data.forEach((item:any) =>{
                    let tempChartSetDataItem:any = {};
                    // 序列属性不存在
                    if(!singleSeries.seriesIdField){
                        Object.assign(tempChartSetDataItem,{name:singleSeries.name});
                        if(singleSeries.dataSetFields && singleSeries.dataSetFields.length >0){
                            singleSeries.dataSetFields.forEach((singleDataSetField:any) =>{
                                this.handleSingleDataSetField(item,singleDataSetField,allCodeList,tempChartSetDataItem);
                            })
                        }
                    }else{
                        // 序列属性存在时
                        let tempSeriesValueItem = tempSeriesValues.get(item[singleSeries.seriesIdField]);
                        if(!tempSeriesValueItem){
                            tempSeriesValues.set(item[singleSeries.seriesIdField],item[singleSeries.seriesNameField]);
                        }
                        Object.assign(tempChartSetDataItem,{name:item[singleSeries.seriesIdField]});
                        if(singleSeries.dataSetFields && singleSeries.dataSetFields.length >0){
                            singleSeries.dataSetFields.forEach((singleDataSetField:any) =>{
                                this.handleSingleDataSetField(item,singleDataSetField,allCodeList,tempChartSetDataItem);
                            })
                        }
                    }
                    tempChartSetData.push(tempChartSetDataItem);
                })
                // 补全代码表
                this.completeCodeList(tempChartSetData,singleSeries,allCodeList);
                singleSeries.seriesValues = [...tempSeriesValues.keys()];
                let tempSeriesMapObj:any = {};
                tempSeriesValues.forEach((value:any,key:any) =>{
                    tempSeriesMapObj[key] = value;
                })
                singleSeries.seriesMap = tempSeriesMapObj;
                let callbackFunction:any = (index === (Object.values(this.seriesModel).length - 1))?callback:null;
                this.transformToChartSeriesDataSet(tempChartSetData,singleSeries,callbackFunction);
            })
        }
    }

    /**
     * 构建图表序列数据集合
     * 
     * @param {Array<any>} data 传入数据
     * @param {Array<any>} item 单个序列
     * @param {Array<any>} callback 回调
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public transformToChartSeriesDataSet(data:any,item:any,callback:Function):any{
        if(item.seriesIdField){
            // 多序列
            let groupField = item.dataSetFields.filter((datasetField:any) =>{
                return datasetField.name === item.categorField;
            });
            let tempGroupField:Array<any> = groupField.map((item:any) =>{
                return item.name;
            });
            let seriesField = item.dataSetFields.filter((datasetField:any) =>{
                return datasetField.name === item.seriesIdField;
            });
            let tempSeriesField:Array<any> = seriesField.map((item:any) =>{
                return item.name;
            });
            let valueField = item.dataSetFields.filter((datasetField:any) =>{
                return datasetField.name === item.valueField;
            });
            let tempValueField:Array<any> = valueField.map((item:any) =>{
                return item.name;
            });
            item.data = this.groupAndAdd(tempGroupField,tempSeriesField,tempValueField,data,item);
        }else{
            //单序列
            let groupField = item.dataSetFields.filter((datasetField:any) =>{
                return datasetField.name === item.categorField;
            });
            let tempGroupField:Array<any> = groupField.map((item:any) =>{
                return item.name;
            })
            let valueField = item.dataSetFields.filter((datasetField:any) =>{
                return datasetField.name === item.valueField;
            });
            let tempValueField:Array<any> = valueField.map((item:any) =>{
                return item.name;
            })
            item.data = this.groupAndAdd(tempGroupField,[],tempValueField,data,item);
        }
        if(callback && callback instanceof Function){
            callback();
        }
    }

    /**
     * 分组和求和  
     * @param {Array<any>} groupField 分组属性
     * @param {Array<any>} groupField 值属性
     * @param {Array<any>} data 传入数据
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public groupAndAdd(groupField:Array<any>,seriesField:Array<any>,valueField:Array<any>,data:any,item:any){
        let tempMap:Map<string,any> = new Map();
        data.forEach((item:any) =>{
            let tempGroupField:string = groupField[0];
            let tempMapItem:any = tempMap.get(item[tempGroupField]);
            if(tempMapItem){
                tempMapItem.push(item);
                tempMap.set(item[tempGroupField],tempMapItem);
            }else{
                tempMap.set(item[tempGroupField],[item]);
            }
        })
        // 处理多序列
        if(seriesField.length > 0 && tempMap.size > 0){
            let tempSeriesField:string = seriesField[0];
            tempMap.forEach((item:any,key:string) =>{
                let tempItemMap:Map<string,any> = new Map();
                item.forEach((singleItem:any)=>{
                    let seriesValueArray:any = tempItemMap.get(singleItem[tempSeriesField]);
                    if(seriesValueArray){
                        seriesValueArray.push(singleItem);
                        tempItemMap.set(singleItem[tempSeriesField],seriesValueArray);
                    }else{
                        tempItemMap.set(singleItem[tempSeriesField],[singleItem]);
                    }
                })
                tempMap.set(key,tempItemMap);
            });
        }
        let returnArray:Array<any> = [];
        if(seriesField.length == 0){
            //单序列
            tempMap.forEach((tempItem:any) =>{
                if(tempItem.length >0){
                    let curObject:any = {};
                    let valueResult:number = 0;
                    let categorResult:any;
                    tempItem.forEach((singleItem:any) =>{
                        categorResult = singleItem[groupField[0]];
                        valueResult += singleItem[valueField[0]];
                    })
                    Object.defineProperty(curObject, groupField[0], {
                        value: categorResult,
                        writable : true,
                        enumerable : true,
                        configurable : true
                    });
                    Object.defineProperty(curObject, valueField[0], {
                        value: valueResult,
                        writable : true,
                        enumerable : true,
                        configurable : true
                    });
                    returnArray.push(curObject);
                }
            })
        }else{
            // 多序列
            let seriesValuesArray:Array<any> = item.seriesValues;
            tempMap.forEach((groupItem:any,groupKey:string) =>{
                //求和
                let curObject:any = {};
                Object.defineProperty(curObject, groupField[0], {
                    value: groupKey,
                    writable : true,
                    enumerable : true,
                    configurable : true
                });
                // valueField.forEach((item:any) =>{
                //     Object.defineProperty(curObject, item, {
                //         value: 0,
                //         writable : true,
                //         enumerable : true,
                //         configurable : true
                //     });
                // });
                seriesValuesArray.forEach((seriesValueItem:any) =>{
                    Object.defineProperty(curObject, seriesValueItem, {
                        value: 0,
                        writable : true,
                        enumerable : true,
                        configurable : true
                    });
                });
                groupItem.forEach((seriesItem:any,seriesKey:string) =>{
                    let seriesNum:number = 0;
                    seriesItem.forEach((dataItem:any) =>{
                        // valueField.forEach((item:any) =>{
                        //     curObject[item] += dataItem[item];
                        // });
                        seriesNum += dataItem[valueField[0]];
                    })
                    curObject[seriesKey] = seriesNum; 
                })
                returnArray.push(curObject);
            })
        }
        console.log(JSON.stringify(returnArray));
        return returnArray;
    }

    /**
     * 补全代码表
     * 
     * @param {Array<any>} data 传入数据
     * @param {Array<any>} item 单个序列
     * @param {Array<any>} allCodeList 所有的代码表
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public  completeCodeList(data:any,item:any,allCodeList:any){
        let groupField = item.dataSetFields.find((datasetField:any) =>{
            return datasetField.name === item.categorField;
        });
        let valueField = item.dataSetFields.find((datasetField:any) =>{
            return datasetField.name === item.valueField;
        });
        if(!groupField.codelist){
            return;
        }
        let curCodeList:Map<number,any> = allCodeList.get(groupField.codelist.tag);
        // 对分类实现分组
        let tempGrounpData:Map<string,any> = new Map();
        data.forEach((item:any) =>{
            if(Object.is(groupField.groupMode,'CODELIST')){
                let tempGrounpItem:any = tempGrounpData.get(item[groupField.name+'_srfvalue']);
                if(!tempGrounpItem){
                    tempGrounpData.set(item[groupField.name+'_srfvalue'],item);
                }
            }else{
                 let tempGrounpItem:any = tempGrounpData.get(item[groupField.name]);
                if(!tempGrounpItem){
                    tempGrounpData.set(item[groupField.name],item);
                }
            }
        })
        if(curCodeList.size !== tempGrounpData.size){
            let copyTemp:any = JSON.parse(JSON.stringify(data[0]));
            curCodeList.forEach((text:any,value:any) =>{
                if(!tempGrounpData.get(value)){
                    let curObj:any = {};
                    if(Object.is(groupField.groupMode,'CODELIST')){
                        Object.defineProperty(curObj, groupField.name+'_srfvalue', {
                            value: value,
                            writable : true,
                            enumerable : true,
                            configurable : true
                        });
                        Object.defineProperty(curObj, groupField.name, {
                            value: text,
                            writable : true,
                            enumerable : true,
                            configurable : true
                        });
                    }else{
                        Object.defineProperty(curObj, groupField.name, {
                            value: value,
                            writable : true,
                            enumerable : true,
                            configurable : true
                        });
                    }
                    Object.defineProperty(curObj, valueField.name, {
                        value: 0,
                        writable : true,
                        enumerable : true,
                        configurable : true
                    });
                    Object.assign(copyTemp,curObj);
                    data.push(copyTemp);
                }
            })
        }  
    }

    /**
     * 处理单个属性
     * 
     * @param {*} input 输入值
     * @param {*} field 属性值
     * @param {*} allCodeList 所有代码表
     * @param {*} result 结果值
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public handleSingleDataSetField(input:any,field:any,allCodeList:any,result:any){
        let tempFieldObj:any = {};
        //存在代码表的情况
        if(field.codelist && Object.is(field.groupMode,"CODELIST")){
            //获取代码表
            let curCodeList:Map<number,any> = allCodeList.get(field.codelist.tag);
            Object.defineProperty(tempFieldObj, field.name, {
                value: curCodeList.get(input[field.name]),
                writable : true,
                enumerable : true,
                configurable : true
            });
            Object.defineProperty(tempFieldObj, field.name+'_srfvalue', {
                value: input[field.name],
                writable : true,
                enumerable : true,
                configurable : true
            });
        }else{
            // 不存在代码表的情况
            Object.defineProperty(tempFieldObj, field.name, {
                value: input[field.name],
                writable : true,
                enumerable : true,
                configurable : true
            });
        }
        Object.assign(result,tempFieldObj);
    }

    /**
     * 获取图表所需代码表
     * 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public async getChartAllCodeList(){
        let codeListMap:Map<string,any> = new Map();
        if(Object.values(this.seriesModel).length >0){
            await Object.values(this.seriesModel).forEach(async (singleSeries:any) =>{
                if(singleSeries.dataSetFields && singleSeries.dataSetFields.length >0){
                    await singleSeries.dataSetFields.forEach(async (singleDataSetField:any) =>{
                        if(singleDataSetField.codelist){
                            if(!codeListMap.get(singleDataSetField.codelist.tag)){
                                let tempCodeListMap:Map<number,any> = new Map();
                                let res:any = await this.getCodeList(singleDataSetField.codelist);
                                if(res && res.length >0){
                                    res.forEach((codeListItem:any) =>{
                                        tempCodeListMap.set(Number(codeListItem.value),codeListItem.text);
                                    })
                                }
                                codeListMap.set(singleDataSetField.codelist.tag,tempCodeListMap);
                            }
                        }
                    })
                }
            })
        }
        return codeListMap;
    }

    /**
     * 获取代码表
     * 
     * @returns {Promise<any>} 
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public getCodeList(codeListObject:any):Promise<any>{
        return new Promise((resolve:any,reject:any) =>{
            if(codeListObject.tag && Object.is(codeListObject.type,"STATIC")){
                const codelist = this.$store.getters.getCodeList(codeListObject.tag);
                if (codelist) {
                    resolve([...JSON.parse(JSON.stringify(codelist.items))]);
                } else {
                    console.log(`<#noparse>----${codeListObject.tag}----代码表不存在</#noparse>`);
                }
            }else if(codeListObject.tag && Object.is(codeListObject.type,"DYNAMIC")){
                this.codeListService.getItems(codeListObject.tag).then((res:any) => {
                    resolve(res);
                }).catch((error:any) => {
                    console.log(`<#noparse>----${codeListObject.tag}----代码表不存在</#noparse>`);
                });
            }
        })
    }


<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>